package com.jiuqi.util.hardware;

import java.util.ArrayList;
import java.util.List;
import com.jiuqi.util.SortedList;
import java.util.StringTokenizer;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.io.UnsupportedEncodingException;
import com.jiuqi.util.StringUtils;
import com.jiuqi.util.JqLib;
import java.security.MessageDigest;

public final class HardwareUtil
{
    private static final int CODE_SEED = 151;
    
    public static final String getMachineCode(final String softFlag) throws Exception {
        final String code = String.valueOf(softFlag) + "@" + getMacAddress();
        final MessageDigest md5 = MessageDigest.getInstance("md5");
        byte[] data = md5.digest(code.getBytes());
        data = encodeData(data);
        final String md5Data = JqLib.bytesToHexString(data);
        String machineCode = "";
        int i;
        for (i = 0, i = 0; i < md5Data.length() - 4; i += 4) {
            machineCode = String.valueOf(machineCode) + md5Data.substring(i, i + 4) + "-";
        }
        machineCode = String.valueOf(machineCode) + md5Data.substring(i, md5Data.length());
        return machineCode;
    }
    
    public static final byte[] getMachineSign(final String category) {
        String macAddr;
        try {
            macAddr = getMacAddress();
            if (StringUtils.isEmpty(macAddr)) {
                return null;
            }
        }
        catch (final Exception ex) {
            return null;
        }
        final String code = String.valueOf(category) + "@" + macAddr;
        byte[] codeBytes;
        try {
            codeBytes = code.getBytes("UTF-8");
        }
        catch (final UnsupportedEncodingException ex2) {
            codeBytes = code.getBytes();
        }
        MessageDigest md5;
        try {
            md5 = MessageDigest.getInstance("MD5");
        }
        catch (final NoSuchAlgorithmException ex3) {
            return null;
        }
        final byte[] data = md5.digest(codeBytes);
        return encodeData(data);
    }
    
    private static byte[] encodeData(final byte[] data) {
        if (data == null) {
            return null;
        }
        final int newSize = data.length / 2;
        final byte[] result = new byte[newSize];
        for (int i = 0; i < newSize; ++i) {
            final int a = data[i * 2];
            final int b = data[i * 2 + 1];
            result[i] = (byte)(a ^ 151 + b ^ 0xFFFFFF68);
        }
        return result;
    }
    
    public static String getMacAddress() throws Exception {
        final String os = System.getProperty("os.name");
        if (os.startsWith("Windows")) {
            return getWindowsMacAddresses();
        }
        if (os.startsWith("Linux")) {
            return getLinuxMacAddresses();
        }
        if (os.startsWith("HP-UX")) {
            return getHPUXMacAddresses();
        }
        return "";
    }
    
    private static String getWindowsMacAddresses() throws IOException {
        final String ipCommandResponse = windowsRunIpConfigCommand();
        return windowsParseMacAddress(ipCommandResponse);
    }
    
    private static String windowsRunIpConfigCommand() throws IOException {
        final Process p = Runtime.getRuntime().exec("ipconfig /all");
        return readInputText(p);
    }
    
    private static String readInputText(final Process p) throws IOException {
        final StringBuffer buffer = new StringBuffer();
        final BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));
        String line;
        while ((line = reader.readLine()) != null) {
            buffer.append(line);
            buffer.append(StringUtils.LINE_SEPARATOR);
        }
        return buffer.toString();
    }
    
    private static String windowsParseMacAddress(final String ipConfigResponse) {
        final StringTokenizer stringTokenizer = new StringTokenizer(ipConfigResponse, "\n");
        final SortedList buffer = new SortedList();
        while (stringTokenizer.hasMoreTokens()) {
            final String line = stringTokenizer.nextToken().trim();
            if (line.startsWith("Physical") || line.startsWith("物理地址")) {
                final int p = line.lastIndexOf(58);
                final String macAddr = (p + 1 >= line.length()) ? null : line.substring(p + 1).trim();
                if (StringUtils.isEmpty(macAddr) || buffer.contains((Object)macAddr)) {
                    continue;
                }
                buffer.add((Object)macAddr);
            }
        }
        return catMacAddrs((List)buffer);
    }
    
    private static String catMacAddrs(final List buffer) {
        if (buffer.size() == 0) {
            return null;
        }
        final StringBuffer result = new StringBuffer(buffer.get(0));
        for (int i = 1; i < buffer.size(); ++i) {
            result.append(';').append(buffer.get(i).toUpperCase());
        }
        return result.toString();
    }
    
    private static String getLinuxMacAddresses() throws Exception {
        final String ifCommandResponse = linuxRunIfConfigCommand();
        return linuxParseMacAddress(ifCommandResponse);
    }
    
    private static String linuxRunIfConfigCommand() throws IOException {
        final Process p = Runtime.getRuntime().exec("ifconfig");
        return readInputText(p);
    }
    
    private static String linuxParseMacAddress(final String ifCommandResponse) {
        final StringTokenizer stringTokenizer = new StringTokenizer(ifCommandResponse, "\n");
        final SortedList buffer = new SortedList();
        while (stringTokenizer.hasMoreTokens()) {
            final String line = stringTokenizer.nextToken();
            final int macPos = line.indexOf("HWaddr");
            if (macPos != -1) {
                final String macAddr = line.substring(macPos + 1).trim();
                if (macAddr.length() != 17 || buffer.contains((Object)macAddr)) {
                    continue;
                }
                buffer.add((Object)macAddr);
            }
        }
        return catMacAddrs((List)buffer);
    }
    
    private static String getHPUXMacAddresses() throws IOException {
        final String lanscanCommandResponse = hpUXRunLanscanCommmand();
        return hpUXParseMacAddress(lanscanCommandResponse);
    }
    
    private static String hpUXRunLanscanCommmand() throws IOException {
        final Process p = Runtime.getRuntime().exec("lanscan");
        return readInputText(p);
    }
    
    private static String hpUXParseMacAddress(final String lanscanCommandResponse) {
        final StringTokenizer stringTokenizer = new StringTokenizer(lanscanCommandResponse, "\n");
        final ArrayList buffer = new ArrayList();
        while (stringTokenizer.hasMoreTokens()) {
            final String line = stringTokenizer.nextToken().trim();
            final int index = line.indexOf("0x");
            if (index != -1) {
                final String macAddr = line.substring(index + 2, index + 2 + 12);
                if (buffer.contains(macAddr)) {
                    continue;
                }
                buffer.add(macAddr);
            }
        }
        return catMacAddrs(buffer);
    }
}
